#
# coding=utf-8
#*********************************************************************
#
# PyMPDTouchGui
# Gui for controlling the MusicPlayDaemon on Touchscreens
#
# written by Timo Boettcher
# pympdtouchgui (near) spida (.) net
#
# (C) 2007-2009 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************



#==============================================================================
class Event:
	def __init__(self):
		self.name = "Generic Event"

#==============================================================================
class TickEvent(Event):
	def __init__(self):
		self.name = "CPU Tick Event"
	def __str__(self):
		return self.name

#==============================================================================
class DisplayUpdateBlinkEvent(Event):
	def __init__(self):
		self.name = "Display Update Blink Event"
	def __str__(self):
		return self.name

#==============================================================================
class ServerUpdateRequestEvent(Event):
	def __init__(self):
		self.name = "Server Update Request Event"
	def __str__(self):
		return self.name

#==============================================================================
class WindowResizeEvent(Event):
	def __init__(self, size):
		self.name = "Window Resize Event"
		self.size = size
	def __str__(self):
		return "Window Resize Event (%s, %s)"%self.size

#==============================================================================
class QuitEvent(Event):
	def __init__(self):
		self.name = "Program Quit Event"
	def __str__(self):
		return self.name

#==============================================================================
class MouseClickEvent(Event):
	def __init__(self, button, clickpos):
		self.name = "Mouse Click Event"
		self.button = button
		self.clickpos = clickpos
	def __str__(self):
		return "%s: (%s, %s)"%(self.name,self.clickpos[0],self.clickpos[1])

#==============================================================================
class KeyboardEvent(Event):
	def __init__(self, key, modifiers):
		self.name = "Keyboard Event"
		self.key = key
		self.modifiers = modifiers
	def __str__(self):
		return "%s: (%s, %s)"%(self.name,self.key)

#==============================================================================
class InfraredEvent(Event):
	def __init__(self, button, repeat):
		self.name = "InfraredEvent"
		self.button = button
		self.repeat = repeat
	def __str__(self):
		return "%s: (%s, %s)"%(self.name,self.button,self.repeat)

#==============================================================================
class PlayUpdateEvent(Event):
	def __init__(self, state, time, random, repeat, volume):
		self.name = "Play Update Event"
		self.state = state
		self.time = time
		self.random = random
		self.repeat = repeat
		self.volume = volume
	def __str__(self):
		return "%s: %s %s"%(self.name,self.status,self.currentsong)

#==============================================================================
class CurrentSongInfoRequestEvent(Event):
	def __init__(self):
		self.name = "Current Song Info Request Event"
	def __str__(self):
		return self.name

#==============================================================================
class CurrentSongInfoEvent(Event):
	def __init__(self, Title, Album, Artist, ChangedTitle, ChangedAlbum, ID):
		self.name = "Current Song Info Event"
		self.Title = Title
		self.Album = Album
		self.Artist = Artist
		self.ChangedTitle = ChangedTitle
		self.ChangedAlbum = ChangedAlbum
		self.ID = ID
	def __str__(self):
		return "%s: %s - %s - %s"%(self.name, self.Album, self.Artist, self.Title)

#==============================================================================
class SearchEvent(Event):
	def __init__(self,searchtype,searchstring):
		self.name = "Search Event"
		self.searchtype = searchtype
		self.searchstring = searchstring
	def __str__(self):
		return self.name + ": " + self.searchtype + " = " + self.searchstring

#==============================================================================
class SearchResultEvent(Event):
	def __init__(self,searchtype,searchstring,searchresults):
		self.name = "Search Result Event"
		self.searchtype = searchtype
		self.searchstring = searchstring
		self.searchresults = searchresults
	def __str__(self):
		return self.name + ": " + self.searchtype + " = " + self.searchstring

#==============================================================================
class PlaylistEvent(Event):
	def __init__(self):
		self.name = "Playlist Event"
	def __str__(self):
		return self.name

#==============================================================================
class PlaylistResultEvent(Event):
	def __init__(self,playlistresults):
		self.name = "Playlist Result Event"
		self.playlistresults = playlistresults
	def __str__(self):
		return self.name

#==============================================================================
class BrowseEvent(Event):
	def __init__(self,path):
		self.name = "Browse Event"
		self.path = path
	def __str__(self):
		return self.name + ": " + self.path

#==============================================================================
class BrowseResultEvent(Event):
	def __init__(self,path,browseresults):
		self.name = "Browse Result Event"
		self.path = path
		self.browseresults = browseresults
	def __str__(self):
		return "%s: %s - %s"%(self.name,self.path,self.browseresults)

#==============================================================================
class InfoareaEvent(Event):
	def __init__(self,InfoEnabled):
		self.name = "Infoarea Event"
		self.InfoEnabled = InfoEnabled
	def __str__(self):
		return self.name

#==============================================================================
class PlaylistAddEvent(Event):
	def __init__(self,path):
		self.name = "Playlist Add Event"
		self.path = path
	def __str__(self):
		return self.name + ": " + self.path

#==============================================================================
class PlaylistClearEvent(Event):
	def __init__(self):
		self.name = "Playlist Clear Event"
	def __str__(self):
		return self.name

#==============================================================================
class PlaylistDelEvent(Event):
	def __init__(self,id):
		self.name = "Playlist Del Event"
		self.id = id
	def __str__(self):
		return self.name

#==============================================================================
class PlaylistMoveEvent(Event):
	def __init__(self,moveid,insertbeforeid):
		self.name = "Playlist Move Event"
		self.moveid = moveid
		self.insertbeforeid = insertbeforeid
	def __str__(self):
		return "Playlist Move Event: Move %s before %s"%(self.moveid, self.insertbeforeid)

#==============================================================================
class MainAreaEvent(Event):
	def __init__(self,widget,extended=None):
		self.name = "Main Area Event"
		self.widget = widget
		self.extended = extended
	def __str__(self):
		return self.name

#==============================================================================
class ScreensaverEvent(Event):
	def __init__(self,ScreensaverEnable=True):
		self.name = "Screensaver Activation Event"
		self.ScreensaverEnable = ScreensaverEnable
	def __str__(self):
		return self.name

#==============================================================================
class ScreensaverChangeEvent(Event):
	def __init__(self):
		self.name = "Screensaver Change Event"
	def __str__(self):
		return self.name

#==============================================================================
class PreviousEvent(Event):
	def __init__(self):
		self.name = "Previous Event"
	def __str__(self):
		return self.name

#==============================================================================
class StopEvent(Event):
	def __init__(self):
		self.name = "Stop Event"
	def __str__(self):
		return self.name

#==============================================================================
class PlayEvent(Event):
	def __init__(self):
		self.name = "Play Event"
	def __str__(self):
		return self.name

#==============================================================================
class PauseEvent(Event):
	def __init__(self):
		self.name = "Pause Event"
	def __str__(self):
		return self.name

#==============================================================================
class NextEvent(Event):
	def __init__(self):
		self.name = "Next Event"
	def __str__(self):
		return self.name

#==============================================================================
class RandomEvent(Event):
	def __init__(self,Random):
		self.name = "Random Event"
		self.Random = Random
	def __str__(self):
		return self.name

#==============================================================================
class RepeatEvent(Event):
	def __init__(self,Repeat):
		self.name = "Repeat Event"
		self.Repeat = Repeat
	def __str__(self):
		return self.name

#==============================================================================
class ReloadConfigEvent(Event):
	def __init__(self):
		self.name = "ReloadConfig Event"
	def __str__(self):
		return self.name

#==============================================================================
class VolumeQueryEvent(Event):
	def __init__(self):
		self.name = "VolumeQuery Event"
	def __str__(self):
		return self.name

#==============================================================================
class VolumeStatusEvent(Event):
	def __init__(self, Volume):
		self.name = "VolumeStatus Event"
		self.volume = Volume
	def __str__(self):
		return self.name

#==============================================================================
class VolumeSetEvent(Event):
	def __init__(self, Volume):
		self.name = "VolumeSet Event"
		self.volume = Volume
	def __str__(self):
		return self.name

#==============================================================================
class VolumeDownEvent(Event):
	def __init__(self):
		self.name = "VolumeDown Event"
	def __str__(self):
		return self.name

#==============================================================================
class MuteEvent(Event):
	def __init__(self):
		self.name = "Mute Event"
	def __str__(self):
		return self.name

#==============================================================================
class VolumeUpEvent(Event):
	def __init__(self):
		self.name = "VolumeUp Event"
	def __str__(self):
		return self.name

#==============================================================================
class UpdateDatabaseEvent(Event):
	def __init__(self):
		self.name = "Update DatabaseEvent"
	def __str__(self):
		return self.name

#==============================================================================
class PowerEvent(Event):
	def __init__(self, PowerDown=True):
		self.name = "PowerEvent"
		self.PowerDown = PowerDown
	def __str__(self):
		if self.PowerDown:
			return "PowerDownEvent"
		else:
			return "PowerUpEvent"
