#
# coding=utf-8
#*********************************************************************
#
# PyMPDTouchGui
# Gui for controlling the MusicPlayDaemon on Touchscreens
#
# written by Timo Boettcher
# pympdtouchgui (near) spida (.) net
#
# (C) 2007-2009 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************


try:
	import sys
	import os
	import string
	import logging
	import pygame
	from pygame import image
	import pygame.locals
	import tempfile
	import datetime
	import random
	import events
	import albumart
	import widgets
except ImportError, err:
	print "couldn't load module. %s"%(err)
	sys.exit(2)


#==============================================================================
class SongArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		TextAreaHeigth = int(Size[1]/2)
		self.LastImageLoad = {}
		self.LastImageLoad["Artist"] = ""
		self.LastImageLoad["Album"] = ""

		Font = pygame.font.Font(self.Config.FontName, self.Config.Sizes.FontSizeStatus)
		LineHeight = Font.get_linesize()
		Spacing = int((TextAreaHeigth - (3 * LineHeight))/4)

		Pos = (Position[0], Position[1] + 1 * Spacing + 0 * LineHeight)
		self.TextArtist = widgets.ScrollText(self, Pos, (Size[0], LineHeight), Config, self.Config.Sizes.FontSizeStatus, self.Config.Theme.ColorForegroundStatus)
		self.TextArtist.TextScroll = True
		self.TextArtist.TextScrollDirectionForwardOnly = False
		self.Widgets.append(self.TextArtist)

		Pos = (Position[0], Position[1] + 2 * Spacing + 1 * LineHeight)
		self.TextAlbum = widgets.ScrollText(self, Pos, (Size[0], LineHeight), Config, self.Config.Sizes.FontSizeStatus, self.Config.Theme.ColorForegroundStatus)
		self.TextAlbum.TextScroll = True
		self.TextAlbum.TextScrollDirectionForwardOnly = False
		self.Widgets.append(self.TextAlbum)

		Pos = (Position[0], Position[1] + 3 * Spacing + 2 * LineHeight)
		self.TextTitle = widgets.ScrollText(self, Pos, (Size[0], LineHeight), Config, self.Config.Sizes.FontSizeStatus, self.Config.Theme.ColorForegroundStatus)
		self.TextTitle.TextScroll = True
		self.TextTitle.TextScrollDirectionForwardOnly = False
		self.Widgets.append(self.TextTitle)


		AlbumArtAreaHeigth = Size[1] - TextAreaHeigth
		AlbumArtSize = int(0.9 * AlbumArtAreaHeigth)
		Pos = (int(Position[0] + Size[0]/2 - AlbumArtSize/2), int(Position[1] + TextAreaHeigth + AlbumArtSize*0.05))
		if self.Config.AlbumArtFromAmazon:
			self.ImageAlbumArt = widgets.ImageButton(self, Pos, (AlbumArtSize,AlbumArtSize), Config, "")
			self.ImageAlbumArt.set_Callback(self.AlbumArtClicked)
		else:
			self.ImageAlbumArt = widgets.Image(self, Pos, (AlbumArtSize,AlbumArtSize), Config)
		self.Widgets.append(self.ImageAlbumArt)


	def HandleEvent(self, event):
		if isinstance(event, events.CurrentSongInfoEvent):
			self.TextArtist.set_Text(event.Artist)
			self.TextAlbum.set_Text(event.Album)
			self.TextTitle.set_Text(event.Title)
			if self.LastImageLoad["Artist"] != event.Artist or self.LastImageLoad["Album"] != event.Album:
				self.LastImageLoad["Artist"] = event.Artist
				self.LastImageLoad["Album"] = event.Album
				self.AlbumArtFile = self.AlbumArtFilename(event.Artist,event.Album)
				if event.Album != "":
					if self.Config.AlbumArtFromAmazon:
						if not os.path.isfile(self.AlbumArtFile):
							AlbumArt = albumart.albumart(self.Config.AmazonID)
							result = AlbumArt.GetAlbumArt(event.Artist, event.Album, self.AlbumArtFile)
							if len(result)==2:
								(imgfound, asin) = result
								self.AlbumartUrl = "http://www.amazon.com/dp/" + asin
							else:
								imgfound = False
								self.AlbumartUrl = ""
							if imgfound:
								self.logger.debug("Fetching Albumart for %s - %s.... Success in %s"%(event.Artist, event.Album, self.AlbumArtFile))
							else:
								self.logger.debug("Fetching Albumart for %s - %s.... Failure"%(event.Artist, event.Album))
								self.AlbumArtFile = ""
						else:
							self.logger.debug("Albumart for %s - %s already exists in %s"%(event.Artist, event.Album, self.AlbumArtFile))
				self.ImageAlbumArt.set_Image(self.AlbumArtFile)

	def AlbumArtClicked(self):
		command = self.Config.Browser
		exe = os.path.split(self.Config.Browser)[1]
		result = os.spawnlp(os.P_NOWAIT, command, exe, self.AlbumartUrl)

	def AlbumArtFilename(self,Artist,Album):
		if Artist != "" and Album != "":
			if self.Config.AlbumArtDir:
				if os.path.isdir(self.Config.AlbumArtDir):
					filename = Artist + " - " + Album + ".jpg"
					return os.path.join(self.Config.AlbumArtDir, filename)
				else:
					filename = "_" + Artist.replace(" ","_") + "_" + Album.replace(" ","_") + ".jpg"
					dir = tempfile.gettempdir()
					return tempfile.mktemp(filename, "pympdtouchgui_",dir)
			else:
				return None
		else:
			return None

#==============================================================================
class InfoArea(widgets.Window):
	MainAreaWidget = False

	def __init__(self, Parent, Position, Size, Config, evManager):
		widgets.Window.__init__(self, Parent, Position, Size, Config)
		self.logger = logging.getLogger('pympdtouchgui')

		self.Enabled = False
		self.Artist = ""
		self.Title = ""

		self.TextArtist = widgets.ScrollText(self, Position, (Size[0], Size[1]/2), Config, self.Config.Sizes.FontSizeInfoArea, self.Config.Theme.ColorForegroundInfo)
		self.TextArtist.TextScroll = True
		self.TextArtist.TextScrollDirectionForwardOnly = False
		self.Widgets.append(self.TextArtist)

		self.TextTitle = widgets.ScrollText(self, (Position[0],Position[1]+Size[1]/2), (Size[0], Size[1]/2), Config, self.Config.Sizes.FontSizeInfoArea, self.Config.Theme.ColorForegroundInfo)
		self.TextTitle.TextScroll = True
		self.TextTitle.TextScrollDirectionForwardOnly = False
		self.Widgets.append(self.TextTitle)

	def HandleEvent(self, event):
		if isinstance(event, events.CurrentSongInfoEvent):
			self.Artist = event.Artist
			self.Title = event.Title
			self.write()
		elif isinstance(event, events.InfoareaEvent):
			self.Enabled = event.InfoEnabled
			self.write()

	def write(self):
		if self.Enabled:
			self.TextArtist.set_Text(self.Artist)
			self.TextTitle.set_Text(self.Title)
		else:
			self.TextArtist.set_Text("")
			self.TextTitle.set_Text("")

#==============================================================================
class PlayControlArea(widgets.Window):
	MainAreaWidget = False
	HashedWidgets = {}

	def __init__(self, Parent, Position, Size, Config, evManager):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)
		self.logger = logging.getLogger('pympdtouchgui')

		Buttons = [("Previous","Previous",self.CallbackPrevious),
			("Stop","Stop",self.CallbackStop),
			("PlayPause","Play",self.CallbackPlay),
			("Next","Next",self.CallbackNext),
			("RandomRepeat","NoRandomNoRepeat",self.CallbackRandomRepeat),
			("VolumeDown","VolumeDown",self.CallbackVolumeDown),
			("Mute","Mute",self.CallbackMute),
			("VolumeUp","VolumeUp",self.CallbackVolumeUp)]
		if Size[0] > Size[1]: # assume horizontal
			ButtonSize = min(Size[1],int(Size[0]/len(Buttons)))
			ButtonsSize = len(Buttons) * ButtonSize
			spacing = int((Size[0] - ButtonsSize) / (len(Buttons) + 1))
			leftoffset = ButtonSize + spacing
			topoffset = 0
			left = Position[0] + spacing
			top = Position[1]
		else: #vertical
			ButtonSize = min(Size[0],int(Size[1]/len(Buttons)))
			ButtonsSize = len(Buttons) * ButtonSize
			spacing = int((Size[1] - ButtonsSize) / (len(Buttons) + 1))
			leftoffset = 0
			topoffset = ButtonSize + spacing
			left = Position[0]
			top = Position[1] + spacing

		for Button in Buttons:
			(Identifier,ButtonType,Callback) = Button
			ButtonWidget = widgets.ImageButton(self, (left,top), (ButtonSize, ButtonSize), Config, ButtonType)
			ButtonWidget.set_Callback(Callback)
			self.HashedWidgets[Identifier] = ButtonWidget
			self.Widgets.append(ButtonWidget)
			left +=leftoffset
			top += topoffset

	def CallbackPrevious(self):
		ev = events.PreviousEvent()
		self.evManager.Notify(ev)

	def CallbackStop(self):
		ev = events.StopEvent()
		self.evManager.Notify(ev)

	def CallbackPlay(self):
		ev = events.PlayEvent()
		self.evManager.Notify(ev)

	def CallbackPause(self):
		ev = events.PauseEvent()
		self.evManager.Notify(ev)

	def CallbackNext(self):
		ev = events.NextEvent()
		self.evManager.Notify(ev)

	def CallbackRandomRepeat(self):
		if self.HashedWidgets["RandomRepeat"].ButtonType == "RandomRepeat":
			ev = events.RandomEvent(False)
			self.evManager.Notify(ev)
			ev = events.RepeatEvent(True)
			self.evManager.Notify(ev)
			self.HashedWidgets["RandomRepeat"].set_ButtonType("NoRandomRepeat")
		elif self.HashedWidgets["RandomRepeat"].ButtonType == "NoRandomRepeat":
			ev = events.RandomEvent(True)
			self.evManager.Notify(ev)
			ev = events.RepeatEvent(False)
			self.evManager.Notify(ev)
			self.HashedWidgets["RandomRepeat"].set_ButtonType("RandomNoRepeat")
		elif self.HashedWidgets["RandomRepeat"].ButtonType == "RandomNoRepeat":
			ev = events.RandomEvent(False)
			self.evManager.Notify(ev)
			ev = events.RepeatEvent(False)
			self.evManager.Notify(ev)
			self.HashedWidgets["RandomRepeat"].set_ButtonType("NoRandomNoRepeat")
		elif self.HashedWidgets["RandomRepeat"].ButtonType == "NoRandomNoRepeat":
			ev = events.RandomEvent(True)
			self.evManager.Notify(ev)
			ev = events.RepeatEvent(True)
			self.evManager.Notify(ev)
			self.HashedWidgets["RandomRepeat"].set_ButtonType("RandomRepeat")
		else:
			self.logger.fatal("Invalid ByuttonType")
			sys.exit(123)

	def CallbackVolumeDown(self):
		ev = events.VolumeDownEvent()
		self.evManager.Notify(ev)

	def CallbackMute(self):
		ev = events.MuteEvent()
		self.evManager.Notify(ev)

	def CallbackVolumeUp(self):
		ev = events.VolumeUpEvent()
		self.evManager.Notify(ev)

	def HandleEvent(self, event):
		if isinstance(event, events.PlayUpdateEvent):
			if event.state == "play":
				self.HashedWidgets["PlayPause"].set_ButtonType("Pause")
				self.HashedWidgets["PlayPause"].set_Callback(self.CallbackPause)
			elif (event.state == "pause") or (event.state == "stop"):
				self.HashedWidgets["PlayPause"].set_ButtonType("Play")
				self.HashedWidgets["PlayPause"].set_Callback(self.CallbackPlay)

			if event.random and event.repeat:
				self.HashedWidgets["RandomRepeat"].set_ButtonType("RandomRepeat")
			elif not event.random and event.repeat:
				self.HashedWidgets["RandomRepeat"].set_ButtonType("NoRandomRepeat")
			elif event.random and not event.repeat:
				self.HashedWidgets["RandomRepeat"].set_ButtonType("RandomNoRepeat")
			elif not event.random and not event.repeat:
				self.HashedWidgets["RandomRepeat"].set_ButtonType("NoRandomNoRepeat")

		elif isinstance(event, events.InfraredEvent):
			ev = None
			if event.button == "Play":
				ev = events.PlayEvent()
			elif event.button == "Pause":
				ev = events.PauseEvent()
			elif event.button == "PlayPause":
				if self.HashedWidgets["PlayPause"].ButtonType == "Play":
					ev = events.PlayEvent()
				elif self.HashedWidgets["PlayPause"].ButtonType == "Pause":
					ev = events.PauseEvent()
			elif event.button == "Stop":
				ev = events.StopEvent()
			elif event.button == "Previous":
				ev = events.PreviousEvent()
			elif event.button == "Next":
				ev = events.NextEvent()
			elif event.button == "Volume-":
				ev = events.VolumeDownEvent()
			elif event.button == "Volume+":
				ev = events.VolumeUpEvent()
			elif event.button == "Mute":
				ev = events.MuteEvent()

			if ev:
				self.evManager.Notify(ev)

#==============================================================================
class MenuStartArea(widgets.Window):
	MainAreaWidget = False

	def __init__(self, Parent, Position, Size, Config, evManager):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		ButtonWidget = widgets.ImageButton(self, Position, Size, Config, "Menu")
		ButtonWidget.set_Callback(self.Callback)
		self.Widgets.append(ButtonWidget)

	def Callback(self):
		ev = events.MainAreaEvent("Menu")
		self.evManager.Notify(ev)

	def HandleEvent(self, event):
		if isinstance(event, events.InfraredEvent):
			if event.button == "Menu":
				ev = events.MainAreaEvent("Menu")
				self.evManager.Notify(ev)

#==============================================================================
class MenuDetailsArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		Buttons = [("Browse","Browse",self.CallbackBrowse),
			("Search","Search",self.CallbackSearch),
			("List","List",self.CallbackList),
			("Status","Status",self.CallbackStatus),
			("AlarmClock","AlarmClock",self.CallbackAlarmClock),
			("SleepClock","SleepClock",self.CallbackSleepClock),
			("Update","Update",self.CallbackUpdate)]
		if self.Config.AllowQuit:
			Buttons.append(("Quit","Quit",self.CallbackQuit))


		ButtonSize = min(Size[1],int(Size[0]/len(Buttons)))
		ButtonsSize = len(Buttons) * ButtonSize
		spacing = int((Size[0] - ButtonsSize) / (len(Buttons) + 1))
		leftoffset = ButtonSize + spacing
		left = Position[0] + spacing
		top = Position[1] + int(Size[1]/2 - ButtonSize/2)

		self.ButtonOrderList = []
		self.MenuButtonActiveIndex = -1

		for Button in Buttons:
			(Identifier,ButtonType,Callback) = Button
			ButtonWidget = widgets.ImageButton(self, (left,top), (ButtonSize, ButtonSize), Config, ButtonType)
			ButtonWidget.set_Callback(Callback)
			self.Widgets.append(ButtonWidget)
			self.ButtonOrderList.append(ButtonWidget)
			left +=leftoffset

	def CallbackBrowse(self, SelectByIR=False):
		ev = events.MainAreaEvent("Browse")
		self.evManager.Notify(ev)

	def CallbackSearch(self, SelectByIR=False):
		extended = {}
		extended["KeyboardIR"]=SelectByIR
		ev = events.MainAreaEvent("Search",extended)
		self.evManager.Notify(ev)

	def CallbackList(self, SelectByIR=False):
		ev = events.MainAreaEvent("List")
		self.evManager.Notify(ev)

	def CallbackStatus(self, SelectByIR=False):
		ev = events.MainAreaEvent("Status")
		self.evManager.Notify(ev)

	def CallbackAlarmClock(self, SelectByIR=False):
		ev = events.MainAreaEvent("AlarmClock")
		self.evManager.Notify(ev)

	def CallbackSleepClock(self, SelectByIR=False):
		ev = events.MainAreaEvent("SleepClock")
		self.evManager.Notify(ev)

	def CallbackUpdate(self, SelectByIR=False):
		ev = events.UpdateDatabaseEvent()
		self.evManager.Notify(ev)
		ev = events.MainAreaEvent("Status")
		self.evManager.Notify(ev)

	def CallbackQuit(self, SelectByIR=False):
		if self.Config.AllowQuit:
			ev = events.QuitEvent()
			self.evManager.Notify(ev)

	def HandleEvent(self, event):
		# FIXME Handle Keyboardinput here, too
		if isinstance(event, events.InfraredEvent):
			if event.button == "Left":
				self.SelectMenuItem(-1)
			elif event.button == "Right":
				self.SelectMenuItem(1)
			elif event.button == "Enter":
				self.ButtonOrderList[self.MenuButtonActiveIndex].CallBack(True)

	def SelectMenuItem(self, relative):
		if self.MenuButtonActiveIndex >= 0 and self.MenuButtonActiveIndex <= len(self.ButtonOrderList) - 1:
			self.ActivateMenuItem(self.ButtonOrderList[self.MenuButtonActiveIndex],False)
		if relative > 0:
			self.MenuButtonActiveIndex = self.MenuButtonActiveIndex + 1
		elif relative <0:
			self.MenuButtonActiveIndex = self.MenuButtonActiveIndex - 1
		if self.MenuButtonActiveIndex < 0:
			self.MenuButtonActiveIndex = len(self.ButtonOrderList) - 1
		elif self.MenuButtonActiveIndex > len(self.ButtonOrderList) - 1:
			self.MenuButtonActiveIndex = 0
		self.ActivateMenuItem(self.ButtonOrderList[self.MenuButtonActiveIndex])

	def ActivateMenuItem(self, MenuItem, Activate=True):
		if Activate:
			if not MenuItem.ButtonType.endswith("Marked"):
				MenuItem.set_ButtonType(MenuItem.ButtonType+"Marked")
		else:
			if MenuItem.ButtonType.endswith("Marked"):
				MenuItem.set_ButtonType(MenuItem.ButtonType[:-6])

#==============================================================================
class KeyboardToggleArea(widgets.Window):
	MainAreaWidget = True
	# Not really a Mainarea Widget, but should be replaced with new Mainareawidget anyway

	def __init__(self, Parent, Position, Size, Config, evManager, extended=None):
		self.KeyboardEnabled = False
		self.KeyboardIR = False
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		if extended != None:
			if isinstance(extended, dict):
				if extended.has_key("KeyboardEnabled"):
					self.KeyboardEnabled = extended["KeyboardEnabled"]
				else:
					self.KeyboardEnabled = False
				if extended.has_key("KeyboardIR"):
					self.KeyboardIR = extended["KeyboardIR"]
				else:
					self.KeyboardIR = False
			else:
				self.logger.fatal("extended is expected to be a dir")
				sys.exit(99)
		self.Callback = None

		self.ToggleKeyboard= widgets.ImageButton(self, Position, Size, Config, "Keyboard")
		self.ToggleKeyboard.set_Callback(self.CallbackToggleKeyboard)
		self.Widgets.append(self.ToggleKeyboard)

	def set_Callback(self, Callback):
		self.Callback = Callback

	def CallbackToggleKeyboard(self):
		self.KeyboardEnabled = not self.KeyboardEnabled
		if self.Callback:
			self.Callback(self.KeyboardEnabled)

	def HandleEvent(self, event):
		pass

#==============================================================================
class BrowseArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager, extended=None):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Windows = []
		self.Path = ""
		self.PathScrollPos = []

		self.KeyboardIR = False
		self.ScrollPos = 0
		KeyboardEnabled = False

		self.CursorWidget = None
		self.CursorLineNumber = -1

		# Print the titlebar with the current path
		font = pygame.font.Font(self.Config.FontName,Config.Sizes.FontSizeBrowse)
		PathHeight = int(font.get_linesize() * 1.3) + 6
		self.PathButton = widgets.TextButton(self, Position, (Size[0],PathHeight), Config, self.Config.Sizes.FontSizeBrowse)
		self.PathButton.set_Text("/")
		self.PathButton.set_Callback(self.CallbackPathButton)
		self.Widgets.append(self.PathButton)

		# prepare the scrollbox
		RowFormat = []
		AddbuttonLength = PathHeight # quadratic Buttons
		LineLength = Config.Sizes.MainAreaSize[0] - AddbuttonLength *2

		RowFormat.append((widgets.TextButton, LineLength))
		RowFormat.append((widgets.ImageButton, AddbuttonLength))

		# now create the scrollbox
		self.ListPosition = (Position[0],Position[1]+PathHeight)
		self.ListLargeSize = (Size[0],Size[1]-PathHeight)
		self.ListSmallSize = (Size[0],int((Size[1]-PathHeight)/2))

		self.vkbdSize = (Size[0],self.ListLargeSize[1]- self.ListSmallSize[1])
		self.vkbdPosition = (Position[0],self.ListPosition[1]+self.ListSmallSize[1])

		if extended != None:
			if isinstance(extended, dict):
				if extended.has_key("KeyboardEnabled"):
					KeyboardEnabled = extended["KeyboardEnabled"]
				if extended.has_key("KeyboardIR"):
					self.KeyboardIR = extended["KeyboardIR"]
				if extended.has_key("ScrollPos"):
					self.ScrollPos = extended["ScrollPos"]
				if extended.has_key("Path"):
					self.Path = extended["Path"]
			else:
				self.logger.fatal("extended is expected to be a dir")
				sys.exit(99)

		if KeyboardEnabled:
			if self.KeyboardIR:
				self.vkbd = widgets.VirtualKeyboardStar(self, self.vkbdPosition, self.vkbdSize, self.Config, self.Config.Sizes.FontSizeKeyboardBrowse)
				self.vkbd.set_CallbackAddText(self.CallbackKeyboard)
			else:
				self.vkbd = widgets.VirtualKeyboardKeyboard(self, self.vkbdPosition, self.vkbdSize, self.Config, self.Config.Sizes.FontSizeKeyboardBrowse)
				self.vkbd.set_CallbackAddText(self.CallbackKeyboard)
			self.Windows.append(self.vkbd)
			self.List = widgets.ScrollableList(self, self.ListPosition, self.ListSmallSize, Config, Config.Sizes.FontSizeBrowse, RowFormat)
		else:
			self.vkbd = None
			self.List = widgets.ScrollableList(self, self.ListPosition, self.ListLargeSize, Config, Config.Sizes.FontSizeBrowse, RowFormat)
		Data = [["",""]]
		self.List.set_Data(Data)
		self.Windows.append(self.List)

		if extended != None:
			ev = events.BrowseEvent(self.Path)
			self.evManager.Notify(ev)

	def CallbackPathButton(self):
		# go back one directory level
		newpath = "/".join(self.Path.split("/")[:-1])
		if len(self.PathScrollPos)>0:
			(self.ScrollPos,self.CursorLineNumber) = self.PathScrollPos.pop()
		self.CursorWidget = None
		ev = events.BrowseEvent(newpath)
		self.evManager.Notify(ev)

	def CallbackDirectory(self,Tag):
		# go into directory
		newpath = Tag
		self.PathScrollPos.append((self.List.Scrollbar.current,self.CursorLineNumber))
		ev = events.BrowseEvent(newpath)
		self.evManager.Notify(ev)

	def CallbackFile(self,Tag):
		pass

	def CallbackAddDirectory(self,Tag):
		# Add whole directory to playlist
		ev = events.PlaylistAddEvent(Tag)
		self.evManager.Notify(ev)

	def CallbackAddFile(self,Tag):
		# Add file to playlist
		ev = events.PlaylistAddEvent(Tag)
		self.evManager.Notify(ev)

	def CallbackToggleKeyboard(self,KeyboardEnabled,KeyboardIR=False):
		extended = {}
		extended["KeyboardIR"] = KeyboardIR
		extended["KeyboardEnabled"] = KeyboardEnabled
		extended["Path"] = self.Path
		extended["ScrollPos"] = self.List.Scrollbar.current
		ev = events.MainAreaEvent("Browse",extended)
		self.evManager.Notify(ev)

	def CallbackKeyboard(self, Key, SwitchOfKeyboard=True):
		if self.Path == "":
			Index = self.List.FindTagStart(0, Key)
		else:
			Index = self.List.FindTagStart(0, self.Path + "/" + Key)
		if Index != None:
			# Scroll in List by using the Scrollbar to keep List and Scrollbar in sync
			self.List.Scrollbar.UpdateScroll(Index)
			self.CursorLineNumber = Index
			# Switch of Keyboard again to make room
			if SwitchOfKeyboard:
				self.CallbackToggleKeyboard(False)

	def HandleEvent(self, event):
		if isinstance(event, events.BrowseResultEvent):
			self.Path = event.path
			Data = []
			for browseresult in event.browseresults:
				if browseresult["type"]=="directory":
					tag = browseresult["directory"]
					text = browseresult["directory"].split("/")[-1]
					RowData = [(text,self.CallbackDirectory,tag),("Add",self.CallbackAddDirectory,tag)]
					Data.append(RowData)
				elif browseresult["type"]=="file":
					tag = browseresult["file"]
					# in case of missing id3-tags, use existing info, fallback to filename, if none exist
					if ((browseresult.has_key("artist")) and (browseresult.has_key("title"))):
						text = browseresult["artist"] + " - " + browseresult["title"]
					elif browseresult.has_key("title"):
						text = browseresult["title"]
					elif browseresult.has_key("artist"):
						text = browseresult["artist"]
					else:
						text = browseresult["file"]
					RowData = [(text,self.CallbackFile,tag),("Add",self.CallbackAddFile,tag)]
					Data.append(RowData)
				else:
					self.logger.warn("Found unknown browseresult type: %s"%browseresult["type"])
			self.List.set_Data(Data)
			self.PathButton.set_Text(self.Path)
			if self.ScrollPos != None:
				self.List.Scrollbar.UpdateScroll(self.ScrollPos)
				if self.CursorLineNumber == -1:
					self.CursorLineNumber = self.ScrollPos
			self.ScrollPos = None
		elif isinstance(event, events.KeyboardEvent):
			if not (event.modifiers & pygame.KMOD_RCTRL) or (event.modifiers & pygame.KMOD_LCTRL) or (event.modifiers & pygame.KMOD_ALT):
				if pygame.key.name(event.key) in string.printable:
					self.CallbackKeyboard(pygame.key.name(event.key),False)
		elif isinstance(event, events.InfraredEvent):
			if event.button == "Left":
				# go back one directory
				self.CallbackPathButton()
				self.SelectLine(0)
			elif event.button == "Right":
				if self.CursorWidget != None:
					if isinstance(self.CursorWidget, widgets.ScrollText):
						self.CursorWidget.CallBack(self.CursorWidget.Tag)
						self.CursorLineNumber = 0
						self.SelectLine(0)
			elif event.button == "Up":
				# scroll up
				self.SelectLine(-1)
			elif event.button == "Down":
				# scroll down
				self.SelectLine(1)
			elif event.button == "PageUp":
				# scroll a page up
				self.SelectLine(-self.List.RowCount)
			elif event.button == "PageDown":
				# scroll a page down
				self.SelectLine(self.List.RowCount)
			elif event.button == "Enter":
				# add item to playlist
				addbutton = self.List.WidgetInDataRow(1, self.CursorLineNumber)
				if addbutton != None:
					addbutton.CallBack(addbutton.Tag)
			elif event.button in ["1","2","3","4","5","6","Cancel"]:
				if not self.KeyboardIR:
					self.CallbackToggleKeyboard(True,True)
				else:
					self.vkbd.CallbackIR(event.button)


	def SelectLine(self, relative):
		# unhighlight the old selected line
		if self.CursorWidget:
			self.HighlightSelect(self.CursorWidget, False)
		# calculate the new selected line
		self.CursorLineNumber = self.CursorLineNumber + relative
		if self.CursorLineNumber > len(self.List.Data) -1:
			self.CursorLineNumber = len(self.List.Data) -1
		if self.CursorLineNumber < 0:
			self.CursorLineNumber = 0
		# scroll to the selected line
		self.List.ScrollToIndex(self.CursorLineNumber - int(self.List.RowCount/2))
		# find the widget for the new seleted line
		self.CursorWidget = self.List.WidgetInDataRow(0, self.CursorLineNumber)
		# highlight the new selected line
		if self.CursorWidget:
			self.HighlightSelect(self.CursorWidget)

	def HighlightSelect(self, widget, highlight=True):
		if isinstance(widget, widgets.ScrollText):
			if highlight:
				widget.BackgroundColor = self.Config.Theme.ColorBackgroundBrowseSelectLine
				widget.ForegroundColor = self.Config.Theme.ColorForegroundBrowseSelectLine
			else:
				widget.BackgroundColor = self.Config.Theme.ColorBackground
				widget.ForegroundColor = self.Config.Theme.ColorForegroundBrowseText
			widget.set_Text(widget.Text)
			widget.Paint()

#==============================================================================
class PlaylistArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Windows = []

		self.ManualScroll = False # Did the user scroll around? If so, don't scroll to the current song.
		self.NowPlayingSongId = 0
		self.NowPlayingWidget = None

		self.CursorWidget = None
		self.CursorLineNumber = -1

		self.SelectedSongTag = None

		# Print the titlebar
		font = pygame.font.Font(self.Config.FontName,Config.Sizes.FontSizeBrowse)
		TitleHeight = int(font.get_linesize() * 1.3) + 6
		self.Title= widgets.TextButton(self, Position, (Size[0]-TitleHeight,TitleHeight), Config, self.Config.Sizes.FontSizeBrowse)
		self.Title.set_Text("Playlist:")
		self.Widgets.append(self.Title)

		self.DelAllButton= widgets.ImageButton(self, (Position[0]+Size[0]-TitleHeight,Position[1]), (TitleHeight,TitleHeight), Config, "Del")
		self.DelAllButton.set_Callback(self.CallbackDelAll)
		self.Widgets.append(self.DelAllButton)

		# prepare the scrollbox
		RowFormat = []
		AddbuttonLength = TitleHeight # quadratic Buttons
		LineLength = Config.Sizes.MainAreaSize[0] - AddbuttonLength *2

		RowFormat.append((widgets.TextButton, LineLength))
		RowFormat.append((widgets.ImageButton, AddbuttonLength))

		# now create the scrollbox
		self.List = widgets.ScrollableList(self, (Position[0],Position[1]+TitleHeight), (Size[0],Size[1]-TitleHeight), Config, Config.Sizes.FontSizeBrowse, RowFormat)
		self.List.set_CallbackScroll(self.CallbackScroll)
		self.List.set_CallbackManualScroll(self.CallbackManualScroll)

		Data = [["",""]]
		self.List.set_Data(Data)

		self.Windows.append(self.List)

	def CallbackManualScroll(self):
		self.ManualScroll = True

	def CallbackScroll(self):
		NowPlayingWidget = self.List.WidgetWithTag(0, self.NowPlayingSongId)
		if NowPlayingWidget:
			self.Highlight(NowPlayingWidget)

	def CallbackDel(self,Tag):
		Scrollpos = self.List.Scrollbar.current
		ev = events.PlaylistDelEvent(int(Tag))
		self.evManager.Notify(ev)
		self.List.ScrollToIndex(Scrollpos)

	def CallbackDelAll(self):
		ev = events.PlaylistClearEvent()
		self.evManager.Notify(ev)

	def CallbackEntry(self,Tag):
		if self.SelectedSongTag == None:
			# No song was selected before, select the song under the cursor
			self.SelectedSongTag = Tag
		else:
			# A song was selected before, insert before song under the cursor
			ev = events.PlaylistMoveEvent(int(self.SelectedSongTag),int(Tag))
			self.SelectedSongTag = None
			self.evManager.Notify(ev)
			ev = events.PlaylistEvent()
			self.evManager.Notify(ev)

	def HandleEvent(self, event):
		if isinstance(event, events.CurrentSongInfoEvent):
			if self.NowPlayingWidget:
				self.Highlight(self.NowPlayingWidget, False)
			if event.ID:
				self.NowPlayingSongId = event.ID
				if not self.ManualScroll:
					# scroll automatically if the user hasn't done so, and set the cursor on the current song
					Index = self.List.FindTag(0, self.NowPlayingSongId)
					if Index != None:
						self.CursorLineNumber = Index
						self.List.ScrollToIndex(Index - int(self.List.RowCount/2))
				self.NowPlayingWidget = self.List.WidgetWithTag(0, self.NowPlayingSongId)
				if self.NowPlayingWidget:
					self.Highlight(self.NowPlayingWidget)
			else:
				self.NowPlayingSongId = None
		elif isinstance(event, events.PlaylistResultEvent):
			if self.NowPlayingWidget:
				self.NowPlayingWidget.BackgroundColor = self.Config.Theme.ColorBackground
			Data = []
			for playlistresult in event.playlistresults:
				if playlistresult.has_key("id"):
					tag = playlistresult["id"]
					# in case of missing id3-tags, use existing info, fallback to filename, if none exist
					if ((playlistresult.has_key("artist")) and (playlistresult.has_key("title"))):
						text = playlistresult["artist"] + " - " + playlistresult["title"]
					elif playlistresult.has_key("title"):
						text = playlistresult["title"]
					elif playlistresult.has_key("artist"):
						text = playlistresult["artist"]
					else:
						text = playlistresult["file"]
					RowData = [(text,self.CallbackEntry,tag),("Del",self.CallbackDel,tag)]
					Data.append(RowData)
			self.List.set_Data(Data)
		elif isinstance(event, events.InfraredEvent):
			if event.button == "Up":
				# scroll up
				self.SelectLine(-1)
			elif event.button == "Down":
				# scroll down
				self.SelectLine(1)
			elif event.button == "PageUp":
				# scroll a page up
				self.SelectLine(-self.List.RowCount)
			elif event.button == "PageDown":
				# scroll a page down
				self.SelectLine(self.List.RowCount)
			elif event.button == "Delete":
				# delete current item
				delbutton = self.List.WidgetInDataRow(1, self.CursorLineNumber)
				if delbutton != None:
					delbutton.CallBack(delbutton.Tag)
			elif event.button == "DeleteAll":
				# delete playlist
				self.CallbackDelAll()

	def Highlight(self, widget, highlight=True):
		if highlight:
			widget.BackgroundColor = self.Config.Theme.ColorBackgroundPlaylistHighlightLine
			widget.ForegroundColor = self.Config.Theme.ColorForegroundPlaylistHighlightLine
		else:
			widget.BackgroundColor = self.Config.Theme.ColorBackground
			widget.ForegroundColor = self.Config.Theme.ColorForegroundPlaylistText
		widget.set_Text(widget.Text)
		widget.Paint()

	def SelectLine(self, relative):
		# unhighlight the old selected line
		if self.CursorWidget:
			self.HighlightSelect(self.CursorWidget, False)
		# calculate the new selected line
		self.CursorLineNumber = self.CursorLineNumber + relative
		if self.CursorLineNumber > len(self.List.Data) -1:
			self.CursorLineNumber = len(self.List.Data) -1
		if self.CursorLineNumber < 0:
			self.CursorLineNumber = 0
		# scroll to the selected line
		self.List.ScrollToIndex(self.CursorLineNumber - int(self.List.RowCount/2))
		self.ManualScroll = True
		# find the widget for the new seleted line
		self.CursorWidget = self.List.WidgetInDataRow(0, self.CursorLineNumber)
		# highlight the new selected line
		if self.CursorWidget:
			self.HighlightSelect(self.CursorWidget)

	def HighlightSelect(self, widget, highlight=True):
		if highlight:
			widget.BackgroundColor = self.Config.Theme.ColorBackgroundPlaylistSelectLine
			widget.ForegroundColor = self.Config.Theme.ColorForegroundPlaylistSelectLine
		else:
			widget.BackgroundColor = self.Config.Theme.ColorBackground
			widget.ForegroundColor = self.Config.Theme.ColorForegroundPlaylistText
		widget.set_Text(widget.Text)
		widget.Paint()

#==============================================================================
class SearchArea(widgets.Window):
	MainAreaWidget = True
	SearchFors = ["Artist","Title","Album"]

	def __init__(self, Parent, Position, Size, Config, evManager, extended=None):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Windows = []

		vkbdSize = (Size[0],int(Size[1]/3*2))
		vkbdPosition = (Position[0],Position[1]+Size[1]-vkbdSize[1])

		font = pygame.font.Font(self.Config.FontName, self.Config.Sizes.FontSizeSearch)

		KeyboardIR = False
		if extended != None:
			if isinstance(extended, dict):
				if extended.has_key("KeyboardIR"):
					KeyboardIR = extended["KeyboardIR"]
			else:
				self.logger.fatal("extended is expected to be a dir")
				sys.exit(99)
		# Calculate Maximum length
		maxsearchforwidth = 0
		for searchfor in self.SearchFors:
			textImg = font.render(searchfor + "jM", self.Config.FontAntialias, (123,123,123)) # ":" and 1 em length, ":" substituted to accomodate for characters with sublength
			if textImg.get_width() > maxsearchforwidth:
				maxsearchforwidth = textImg.get_width()

		searchforOffset = int(Size[0]/50)
		searchforSize = (maxsearchforwidth,Size[1]-vkbdSize[1])
		searchforPosition = (Position[0]+searchforOffset,Position[1])
		self.SearchFor = widgets.TextButton(self, searchforPosition, searchforSize, Config, self.Config.Sizes.FontSizeSearch)
		self.SearchFor.Align = "left"
		self.SearchFor.set_Text(self.SearchFors[0])
		self.SearchFor.set_Callback(self.CallbackSearchFor)
		self.Widgets.append(self.SearchFor)

		inputboxPosition = (Position[0]+searchforPosition[0]+searchforSize[0],Position[1])
		inputboxSize = ((Size[0]-inputboxPosition[0])*0.95,Size[1]-vkbdSize[1])
		self.InputBox = widgets.TextInput(self, inputboxPosition, inputboxSize, Config, self.Config.Sizes.FontSizeSearch)
		self.InputBox.Align = "left"
		self.Widgets.append(self.InputBox)

		if not KeyboardIR:
			self.vkbd = widgets.VirtualKeyboardKeyboard(self, vkbdPosition, vkbdSize, Config, self.Config.Sizes.FontSizeKeyboardSearch)
		else:
			self.vkbd = widgets.VirtualKeyboardStar(self, vkbdPosition, vkbdSize, Config, self.Config.Sizes.FontSizeKeyboardSearch)
		self.vkbd.set_CallbackAddText(self.InputBox.AddText)
		self.vkbd.set_CallbackBackspace(self.InputBox.Backspace)
		self.vkbd.set_Callback(self.CallbackSearch)
		self.Windows.append(self.vkbd)


	def CallbackSearch(self):
		ev = events.MainAreaEvent("SearchResult")
		self.evManager.Notify(ev)
		ev = events.SearchEvent(self.SearchFor.Text,self.InputBox.Text)
		self.evManager.Notify(ev)

	def CallbackSearchFor(self):
		currentindex = self.SearchFors.index(self.SearchFor.Text)
		newindex = currentindex + 1
		if newindex >= len(self.SearchFors):
			newindex = 0
		self.SearchFor.set_Text(self.SearchFors[newindex])


	def HandleEvent(self, event):
		if isinstance(event, events.KeyboardEvent):
			if not (event.modifiers & pygame.KMOD_RCTRL) or (event.modifiers & pygame.KMOD_LCTRL) or (event.modifiers & pygame.KMOD_ALT):
				if pygame.key.name(event.key) == "backspace":
					self.InputBox.set_Text(self.InputBox.Text[:-1])
				elif pygame.key.name(event.key) == "return":
					self.CallbackSearch()
				elif pygame.key.name(event.key) in string.printable:
					if event.modifiers & 1: # CAPITAL LETTER
						self.InputBox.set_Text(self.InputBox.Text + pygame.key.name(event.key).upper())
					else:
						self.InputBox.set_Text(self.InputBox.Text + pygame.key.name(event.key))
		elif isinstance(event, events.DisplayUpdateBlinkEvent):
			self.InputBox.BlinkCursor()
		elif isinstance(event, events.InfraredEvent):
			if event.button in ["1","2","3","4","5","6","Delete","Cancel"]:
				self.vkbd.CallbackIR(event.button)
			elif event.button == "Enter":
				self.CallbackSearch()
			elif event.button == "SearchFor":
				self.CallbackSearchFor()

#==============================================================================
class SearchResultArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Windows = []

		self.CursorWidget = None
		self.CursorLineNumber = -1

		# Print the titlebar with the current path
		font = pygame.font.Font(self.Config.FontName,Config.Sizes.FontSizeSearchResult)
		PathHeight = int(font.get_linesize() * 1.3) + 6
		self.SearchResultTitleButton = widgets.TextButton(self, Position, (Size[0],PathHeight), Config, self.Config.Sizes.FontSizeSearchResult)
		self.SearchResultTitleButton.set_Text("")
		self.SearchResultTitleButton.set_Callback(self.CallbackSearchResultTitle)
		self.Widgets.append(self.SearchResultTitleButton)

		# prepare the scrollbox
		RowFormat = []
		AddbuttonLength = PathHeight # quadratic Buttons
		LineLength = Config.Sizes.MainAreaSize[0] - AddbuttonLength *2

		RowFormat.append((widgets.TextButton, LineLength))
		RowFormat.append((widgets.ImageButton, AddbuttonLength))

		# now create the scrollbox
		self.List = widgets.ScrollableList(self, (Position[0],Position[1]+PathHeight), (Size[0],Size[1]-PathHeight), Config, Config.Sizes.FontSizeSearchResult, RowFormat)

		Data = [["",""]]
		self.List.set_Data(Data)

		self.Windows.append(self.List)

	def CallbackSearchResultTitle(self):
		# Search again
		ev = events.MainAreaEvent("Search")
		self.evManager.Notify(ev)

	def CallbackFile(self,Tag):
		pass

	def CallbackAddFile(self,Tag):
		# Add file to playlist
		ev = events.PlaylistAddEvent(Tag)
		self.evManager.Notify(ev)

	def HandleEvent(self, event):
		if isinstance(event, events.SearchResultEvent):
			self.SearchResultTitleButton.set_Text("%s-search for %s:"%(event.searchtype, event.searchstring))
			Data = []
			for searchresult in event.searchresults:
				tag = searchresult["file"]
				# in case of missing id3-tags, use existing info, fallback to filename, if none exist
				if ((searchresult.has_key("artist")) and (searchresult.has_key("title"))):
					text = searchresult["artist"] + " - " + searchresult["title"]
				elif searchresult.has_key("title"):
					text = searchresult["title"]
				elif searchresult.has_key("artist"):
					text = searchresult["artist"]
				else:
					text = searchresult["file"]
				RowData = [(text,self.CallbackFile,tag),("Add",self.CallbackAddFile,tag)]
				Data.append(RowData)
			self.List.set_Data(Data)
		elif isinstance(event, events.InfraredEvent):
			if event.button == "Left":
				# go back one directory
				self.CallbackPathButton()
			elif event.button == "Right":
				self.CallbackDirectory(self.CursorWidget.Tag)
				self.CursorLineNumber = 0
				self.SelectLine(0)
			elif event.button == "Up":
				# scroll up
				self.SelectLine(-1)
			elif event.button == "Down":
				# scroll down
				self.SelectLine(1)
			elif event.button == "PageUp":
				# scroll a page up
				self.SelectLine(-self.List.RowCount)
			elif event.button == "PageDown":
				# scroll a page down
				self.SelectLine(self.List.RowCount)
			elif event.button == "Enter":
				# add item to playlist
				addbutton = self.List.WidgetInDataRow(1, self.CursorLineNumber)
				if addbutton != None:
					addbutton.CallBack(addbutton.Tag)

	def SelectLine(self, relative):
		# unhighlight the old selected line
		if self.CursorWidget:
			self.HighlightSelect(self.CursorWidget, False)
		# calculate the new selected line
		self.CursorLineNumber = self.CursorLineNumber + relative
		if self.CursorLineNumber > len(self.List.Data) -1:
			self.CursorLineNumber = len(self.List.Data) -1
		if self.CursorLineNumber < 0:
			self.CursorLineNumber = 0
		# scroll to the selected line
		self.List.ScrollToIndex(self.CursorLineNumber - int(self.List.RowCount/2))
		# find the widget for the new seleted line
		self.CursorWidget = self.List.WidgetInDataRow(0, self.CursorLineNumber)
		# highlight the new selected line
		if self.CursorWidget:
			self.HighlightSelect(self.CursorWidget)

	def HighlightSelect(self, widget, highlight=True):
		if highlight:
			widget.BackgroundColor = self.Config.Theme.ColorBackgroundBrowseSelectLine
			widget.ForegroundColor = self.Config.Theme.ColorForegroundBrowseSelectLine
		else:
			widget.BackgroundColor = self.Config.Theme.ColorBackground
			widget.ForegroundColor = self.Config.Theme.ColorForegroundBrowseText
		widget.set_Text(widget.Text)
		widget.Paint()

#==============================================================================
class AlarmClockArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Day = 0
		FontSize = Config.Sizes.FontSizeAlarmClock

		self.ButtonOrderList = []
		self.ButtonOrderActiveItem = -1

		font = pygame.font.Font(self.Config.FontName,FontSize)
		LineHeight = int(font.get_linesize() * 1.3)

		ConfigShowDayIncDec = False
		self.days = ["Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"]
		maxdaywidth = 0
		for day in self.days:
			textImg = font.render(day, self.Config.FontAntialias, (123,123,123))
			if textImg.get_width() > maxdaywidth:
				maxdaywidth = textImg.get_width()
		maxdaywidth = int(maxdaywidth * 1.2)


		# Calculate Maximum length
		maxnumberwidth = 0
		for number in xrange(0,9):
			textImg = font.render(str(number), self.Config.FontAntialias, (123,123,123))
			if textImg.get_width() > maxnumberwidth:
				maxnumberwidth = textImg.get_width()
		maxnumberwidth = int(maxnumberwidth * 1.2)

		# Calculate Sizes
		DaySize     = (maxdaywidth,     LineHeight)
		HourSize    = (maxnumberwidth*2,LineHeight)
		DividerSize = (maxnumberwidth,  LineHeight)
		MinuteSize  = (maxnumberwidth*2,LineHeight)
		EnableSize  = (maxnumberwidth*2,LineHeight)
		IncDecSize = (int(LineHeight/1.3),int(LineHeight/1.3))

		Centerline    = Position[1] + int(Size[1]/4) - int(LineHeight*0.5)
		IncButtonLine = Centerline  - int(IncDecSize[1]/2) - int(LineHeight*0.5)
		DecButtonLine = Centerline  - int(IncDecSize[1]/2) + int(LineHeight*1.5)

		DayPosition     = (Position[0] + int(Size[0]/2 - maxdaywidth - maxnumberwidth*7/2), Centerline)
		HourPosition    = (Position[0] + int(Size[0]/2 - maxnumberwidth*5/2),               Centerline)
		DividerPosition = (Position[0] + int(Size[0]/2 - maxnumberwidth/2),                 Centerline)
		MinutePosition  = (Position[0] + int(Size[0]/2 + maxnumberwidth/2),                 Centerline)
		EnablePosition  = (Position[0] + int(Size[0]/2 + maxnumberwidth*7/2),               Centerline)

		DayIncPosition    = (Position[0] + int(Size[0]/2 - maxdaywidth - maxnumberwidth*5/2 - IncDecSize[0]/2), IncButtonLine)
		DayDecPosition    = (Position[0] + int(Size[0]/2 - maxdaywidth - maxnumberwidth*5/2 - IncDecSize[0]/2), DecButtonLine)
		HourIncPosition   = (Position[0] + int(Size[0]/2 - maxnumberwidth*3/2 - IncDecSize[0]/2), IncButtonLine)
		HourDecPosition   = (Position[0] + int(Size[0]/2 - maxnumberwidth*3/2 - IncDecSize[0]/2), DecButtonLine)
		MinuteIncPosition = (Position[0] + int(Size[0]/2 + maxnumberwidth*3/2 - IncDecSize[0]/2), IncButtonLine)
		MinuteDecPosition = (Position[0] + int(Size[0]/2 + maxnumberwidth*3/2 - IncDecSize[0]/2), DecButtonLine)

		WeekTimePosition = {}
		WeekTextPosition = {}
		WeekEnablePosition = {}

		spacing = int(Size[0]/7)
		WeekTimeSize   = (int(spacing/4*3),int(LineHeight/4*3))
		WeekTextSize   = (int(spacing/4*3)-int(LineHeight/4*3),int(LineHeight/4*3))
		WeekEnableSize = (int(LineHeight/4*3),int(LineHeight/4*3))
		for daynumber,dayname in enumerate(self.days):
			WeekTimePosition[daynumber]   = (spacing*daynumber, Position[1] + int(Size[1]*2/3) + LineHeight)
			WeekTextPosition[daynumber]   = (spacing*daynumber, Position[1] + int(Size[1]*2/3))
			WeekEnablePosition[daynumber] = (spacing*daynumber + WeekTextSize[0], Position[1] + int(Size[1]*2/3))


		# Instantiate Widgets
		self.DayButton = widgets.ScrollText(self, DayPosition, DaySize, self.Config, FontSize)
		self.DayButton.set_Text(self.days[self.Day])
		self.Widgets.append(self.DayButton)

		if ConfigShowDayIncDec:
			self.DayIncrementButton = widgets.ImageButton(self, DayIncPosition, IncDecSize, self.Config, "Increment")
		else:
			self.DayIncrementButton = widgets.ImageButton(self, DayIncPosition, IncDecSize, self.Config, "")
		self.DayIncrementButton.set_Callback(self.CallbackDayIncrementButton)
		self.Widgets.append(self.DayIncrementButton)

		if ConfigShowDayIncDec:
			self.DayDecrementButton = widgets.ImageButton(self, DayDecPosition, IncDecSize, self.Config, "Decrement")
		else:
			self.DayDecrementButton = widgets.ImageButton(self, DayDecPosition, IncDecSize, self.Config, "")
		self.DayDecrementButton.set_Callback(self.CallbackDayDecrementButton)
		self.Widgets.append(self.DayDecrementButton)
		self.ButtonOrderList.append({"Down":self.DayDecrementButton, "Up":self.DayIncrementButton})

		self.HourButton = widgets.ScrollText(self, HourPosition, HourSize, Config, FontSize)
		text = "%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day][0]
		self.HourButton.set_Text(text)
		self.Widgets.append(self.HourButton)

		self.HourIncrementButton = widgets.ImageButton(self, HourIncPosition, IncDecSize, self.Config, "Increment")
		self.HourIncrementButton.set_Callback(self.CallbackHourIncrementButton)
		self.Widgets.append(self.HourIncrementButton)

		self.HourDecrementButton = widgets.ImageButton(self, HourDecPosition, IncDecSize, self.Config, "Decrement")
		self.HourDecrementButton.set_Callback(self.CallbackHourDecrementButton)
		self.Widgets.append(self.HourDecrementButton)
		self.ButtonOrderList.append({"Down":self.HourDecrementButton, "Up":self.HourIncrementButton})

		self.TextDivider = widgets.ScrollText(self, DividerPosition, DividerSize, self.Config, FontSize, self.Config.Theme.ColorForegroundInfo)
		self.TextDivider.set_Text(":")
		self.Widgets.append(self.TextDivider)

		self.MinuteButton = widgets.ScrollText(self, MinutePosition, MinuteSize, Config, FontSize)
		text = "%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day][1]
		self.MinuteButton.set_Text(text)
		self.Widgets.append(self.MinuteButton)

		self.MinuteIncrementButton = widgets.ImageButton(self, MinuteIncPosition, IncDecSize, self.Config, "Increment")
		self.MinuteIncrementButton.set_Callback(self.CallbackMinuteIncrementButton)
		self.Widgets.append(self.MinuteIncrementButton)

		self.MinuteDecrementButton = widgets.ImageButton(self, MinuteDecPosition, IncDecSize, self.Config, "Decrement")
		self.MinuteDecrementButton.set_Callback(self.CallbackMinuteDecrementButton)
		self.Widgets.append(self.MinuteDecrementButton)
		self.ButtonOrderList.append({"Down":self.MinuteDecrementButton, "Up":self.MinuteIncrementButton})

		if self.Config.AlarmClock.AlarmClockEnabled[self.Day]:
			self.EnableButton = widgets.ImageButton(self, EnablePosition, EnableSize, self.Config, "CheckBoxChecked")
		else:
			self.EnableButton = widgets.ImageButton(self, EnablePosition, EnableSize, self.Config, "CheckBoxEmpty")
		self.EnableButton.set_Callback(self.CallbackEnableButton)
		self.Widgets.append(self.EnableButton)
		self.ButtonOrderList.append({"Enter":self.EnableButton})

		self.WeekButtonText={}
		self.WeekButtonTime={}
		self.WeekButtonEnabled={}
		for daynumber,dayname in enumerate(self.days):
			self.WeekButtonText[daynumber] = widgets.TextButton(self, WeekTextPosition[daynumber], WeekTextSize, self.Config, FontSize)
			self.WeekButtonText[daynumber].set_Tag(daynumber)
			self.WeekButtonText[daynumber].set_Callback(self.CallbackWeek)
			self.WeekButtonText[daynumber].set_Text(dayname)
			self.Widgets.append(self.WeekButtonText[daynumber])

			self.WeekButtonTime[daynumber] = widgets.TextButton(self, WeekTimePosition[daynumber], WeekTimeSize, self.Config, FontSize)
			self.WeekButtonTime[daynumber].set_Tag(daynumber)
			self.WeekButtonTime[daynumber].set_Callback(self.CallbackWeek)
			text = "%02u:%02u"%self.Config.AlarmClock.AlarmClockTime[daynumber]
			self.WeekButtonTime[daynumber].set_Text(text)
			self.Widgets.append(self.WeekButtonTime[daynumber])

			if self.Config.AlarmClock.AlarmClockEnabled[daynumber]:
				self.WeekButtonEnabled[daynumber] = widgets.ImageButton(self, WeekEnablePosition[daynumber], WeekEnableSize, self.Config, "CheckBoxChecked")
			else:
				self.WeekButtonEnabled[daynumber] = widgets.ImageButton(self, WeekEnablePosition[daynumber], WeekEnableSize, self.Config, "CheckBoxEmpty")
			self.WeekButtonEnabled[daynumber].set_Tag(daynumber)
			self.WeekButtonEnabled[daynumber].set_Callback(self.CallbackWeek)
			self.Widgets.append(self.WeekButtonEnabled[daynumber])

	def CallbackWeek(self, Tag):
		# Load Data of new day
		self.Day = Tag
		text = "%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day][0]
		self.HourButton.set_Text(text)
		text = "%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day][1]
		self.MinuteButton.set_Text(text)
		self.DayButton.set_Text(self.days[self.Day])
		if self.Config.AlarmClock.AlarmClockEnabled[self.Day]:
			self.EnableButton.set_ButtonType("CheckBoxChecked")
		else:
			self.EnableButton.set_ButtonType("CheckBoxEmpty")

	def CallbackEnableButton(self):
		self.Config.AlarmClock.AlarmClockEnabled[self.Day] = not self.Config.AlarmClock.AlarmClockEnabled[self.Day]
		if self.Config.AlarmClock.AlarmClockEnabled[self.Day]:
			if self.EnableButton.ButtonType.endswith("Marked"):
				self.EnableButton.set_ButtonType("CheckBoxCheckedMarked")
			else:
				self.EnableButton.set_ButtonType("CheckBoxChecked")
			self.WeekButtonEnabled[self.Day].set_ButtonType("CheckBoxChecked")
		else:
			if self.EnableButton.ButtonType.endswith("Marked"):
				self.EnableButton.set_ButtonType("CheckBoxEmptyMarked")
			else:
				self.EnableButton.set_ButtonType("CheckBoxEmpty")
			self.WeekButtonEnabled[self.Day].set_ButtonType("CheckBoxEmpty")


	def CallbackDayIncrementButton(self):
		Day = self.Day + 1
		if Day > 6:
			Day = 0
		self.CallbackWeek(Day)

	def CallbackDayDecrementButton(self):
		Day = self.Day - 1
		if Day < 0:
			Day = 6
		self.CallbackWeek(Day)

	def CallbackHourIncrementButton(self):
		Hour = self.Config.AlarmClock.AlarmClockTime[self.Day][0]
		Minute = self.Config.AlarmClock.AlarmClockTime[self.Day][1]
		Hour = Hour + 1
		if Hour > 23:
			Hour = 0
		text = "%02u"%Hour
		self.HourButton.set_Text(text)
		self.Config.AlarmClock.AlarmClockTime[self.Day] = (Hour,Minute)
		text = "%02u:%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day]
		self.WeekButtonTime[self.Day].set_Text(text)
		self.WeekButtonTime[self.Day].Paint()


	def CallbackHourDecrementButton(self):
		Hour = self.Config.AlarmClock.AlarmClockTime[self.Day][0]
		Minute = self.Config.AlarmClock.AlarmClockTime[self.Day][1]
		Hour = Hour - 1
		if Hour < 0:
			Hour = 23
		text = "%02u"%Hour
		self.HourButton.set_Text(text)
		self.Config.AlarmClock.AlarmClockTime[self.Day] = (Hour,Minute)
		text = "%02u:%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day]
		self.WeekButtonTime[self.Day].set_Text(text)
		self.WeekButtonTime[self.Day].Paint()

	def CallbackMinuteIncrementButton(self):
		Hour = self.Config.AlarmClock.AlarmClockTime[self.Day][0]
		Minute = self.Config.AlarmClock.AlarmClockTime[self.Day][1]
		Minute = Minute + 1
		if Minute > 59:
			Minute = 0
		text = "%02u"%Minute
		self.MinuteButton.set_Text(text)
		self.Config.AlarmClock.AlarmClockTime[self.Day] = (Hour,Minute)
		text = "%02u:%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day]
		self.WeekButtonTime[self.Day].set_Text(text)
		self.WeekButtonTime[self.Day].Paint()

	def CallbackMinuteDecrementButton(self):
		Hour = self.Config.AlarmClock.AlarmClockTime[self.Day][0]
		Minute = self.Config.AlarmClock.AlarmClockTime[self.Day][1]
		Minute = Minute - 1
		if Minute < 0:
			Minute = 59
		text = "%02u"%Minute
		self.MinuteButton.set_Text(text)
		self.Config.AlarmClock.AlarmClockTime[self.Day] = (Hour,Minute)
		text = "%02u:%02u"%self.Config.AlarmClock.AlarmClockTime[self.Day]
		self.WeekButtonTime[self.Day].set_Text(text)
		self.WeekButtonTime[self.Day].Paint()

	def kill(self):
		self.Config.AlarmClock.ConfigFileWrite()
		ev = events.ReloadConfigEvent()
		self.evManager.Notify(ev)
		widgets.Window.kill(self)

	def HandleEvent(self, event):
		if isinstance(event, events.InfraredEvent):
			if event.button == "Left":
				self.SelectMenuItem(-1)
			elif event.button == "Right":
				self.SelectMenuItem(1)
			elif event.button == "Up":
				if self.ButtonOrderList[self.ButtonOrderActiveItem].has_key("Up"):
					self.ButtonOrderList[self.ButtonOrderActiveItem]["Up"].CallBack()
			elif event.button == "Down":
				if self.ButtonOrderList[self.ButtonOrderActiveItem].has_key("Down"):
					self.ButtonOrderList[self.ButtonOrderActiveItem]["Down"].CallBack()
			elif event.button == "Enter":
				if self.ButtonOrderList[self.ButtonOrderActiveItem].has_key("Enter"):
					self.ButtonOrderList[self.ButtonOrderActiveItem]["Enter"].CallBack()

	def SelectMenuItem(self, relative):
		if self.ButtonOrderActiveItem >= 0 and self.ButtonOrderActiveItem <= len(self.ButtonOrderList) - 1:
			self.ActivateMenuItem(self.ButtonOrderList[self.ButtonOrderActiveItem],False)
		if relative > 0:
			self.ButtonOrderActiveItem = self.ButtonOrderActiveItem + 1
		elif relative <0:
			self.ButtonOrderActiveItem = self.ButtonOrderActiveItem - 1
		if self.ButtonOrderActiveItem < 0:
			self.ButtonOrderActiveItem = len(self.ButtonOrderList) - 1
		elif self.ButtonOrderActiveItem > len(self.ButtonOrderList) - 1:
			self.ButtonOrderActiveItem = 0
		self.ActivateMenuItem(self.ButtonOrderList[self.ButtonOrderActiveItem])


	def ActivateMenuItem(self, MenuItems, Activate=True):
		for MenuItemKey in MenuItems.keys():
			MenuItem = MenuItems[MenuItemKey]
			if MenuItem.ButtonType != "":
				if Activate:
					if not MenuItem.ButtonType.endswith("Marked"):
						MenuItem.set_ButtonType(MenuItem.ButtonType+"Marked")
				else:
					if MenuItem.ButtonType.endswith("Marked"):
						MenuItem.set_ButtonType(MenuItem.ButtonType[:-6])
			else:
				if Activate:
					self.DayButton.ForegroundColor = (255,0,0)
				else:
					self.DayButton.ForegroundColor = (255,255,255)
				self.DayButton.set_Text(self.DayButton.Text)

#==============================================================================
class SleepClockArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Day = 0
		FontSize = Config.Sizes.FontSizeSleepClock

		font = pygame.font.Font(self.Config.FontName,FontSize)
		LineHeight = int(font.get_linesize() * 1.3)

		self.ButtonOrderList = []
		self.ButtonOrderActiveItem = -1

		# Calculate Maximum length
		maxnumberwidth = 0
		for number in xrange(0,9):
			textImg = font.render(str(number), self.Config.FontAntialias, (123,123,123))
			if textImg.get_width() > maxnumberwidth:
				maxnumberwidth = textImg.get_width()
		maxnumberwidth = int(maxnumberwidth * 1.2)

		# Calculate Sizes
		HourSize    = (maxnumberwidth*2,LineHeight)
		DividerSize = (maxnumberwidth,  LineHeight)
		MinuteSize  = (maxnumberwidth*2,LineHeight)
		EnableSize  = (maxnumberwidth*2,LineHeight)
		IncDecSize = (int(LineHeight/1.3),int(LineHeight/1.3))

		Centerline    = Position[1] + int(Size[1]/4) - int(LineHeight*0.5)
		IncButtonLine = Centerline  - int(IncDecSize[1]/2) - int(LineHeight*0.5)
		DecButtonLine = Centerline  - int(IncDecSize[1]/2) + int(LineHeight*1.5)

		HourPosition    = (Position[0] + int(Size[0]/2 - maxnumberwidth*5/2), Centerline)
		DividerPosition = (Position[0] + int(Size[0]/2 - maxnumberwidth/2),   Centerline)
		MinutePosition  = (Position[0] + int(Size[0]/2 + maxnumberwidth/2),   Centerline)
		EnablePosition  = (Position[0] + int(Size[0]/2 + maxnumberwidth*7/2), Centerline)

		HourIncPosition   = (Position[0] + int(Size[0]/2 - maxnumberwidth*3/2 - IncDecSize[0]/2), IncButtonLine)
		HourDecPosition   = (Position[0] + int(Size[0]/2 - maxnumberwidth*3/2 - IncDecSize[0]/2), DecButtonLine)
		MinuteIncPosition = (Position[0] + int(Size[0]/2 + maxnumberwidth*3/2 - IncDecSize[0]/2), IncButtonLine)
		MinuteDecPosition = (Position[0] + int(Size[0]/2 + maxnumberwidth*3/2 - IncDecSize[0]/2), DecButtonLine)


		if self.Config.AlarmClock.SleepClockEnabled:
			SleepTime = self.Config.AlarmClock.SleepClockTime
		else:
			# Default Sleep: 30min in the future
			SleepTime = datetime.datetime.now() + datetime.timedelta(0,0,0,0,30)
			# timedelta is (days, seconds, microseconds, milliseconds, minutes, hours, weeks)
		SleepTimeDiff = SleepTime - datetime.datetime.now()
		self.SleepHours   = int(SleepTimeDiff.seconds/3600)
		self.SleepMinutes = int((SleepTimeDiff.seconds - self.SleepHours*3600) /60)


		# Instantiate Widgets
		self.HourButton = widgets.ScrollText(self, HourPosition, HourSize, Config, FontSize)
		text = "%02u"%self.SleepHours
		self.HourButton.set_Text(text)
		self.Widgets.append(self.HourButton)

		self.HourIncrementButton = widgets.ImageButton(self, HourIncPosition, IncDecSize, self.Config, "Increment")
		self.HourIncrementButton.set_Callback(self.CallbackHourIncrementButton)
		self.Widgets.append(self.HourIncrementButton)

		self.HourDecrementButton = widgets.ImageButton(self, HourDecPosition, IncDecSize, self.Config, "Decrement")
		self.HourDecrementButton.set_Callback(self.CallbackHourDecrementButton)
		self.Widgets.append(self.HourDecrementButton)
		self.ButtonOrderList.append({"Down":self.HourDecrementButton, "Up":self.HourIncrementButton})

		self.TextDivider = widgets.ScrollText(self, DividerPosition, DividerSize, Config, FontSize, self.Config.Theme.ColorForegroundInfo)
		self.TextDivider.set_Text(":")
		self.Widgets.append(self.TextDivider)

		self.MinuteButton = widgets.ScrollText(self, MinutePosition, MinuteSize, Config, FontSize)
		text = "%02u"%self.SleepMinutes
		self.MinuteButton.set_Text(text)
		self.Widgets.append(self.MinuteButton)

		self.MinuteIncrementButton = widgets.ImageButton(self, MinuteIncPosition, IncDecSize, self.Config, "Increment")
		self.MinuteIncrementButton.set_Callback(self.CallbackMinuteIncrementButton)
		self.Widgets.append(self.MinuteIncrementButton)

		self.MinuteDecrementButton = widgets.ImageButton(self, MinuteDecPosition, IncDecSize, self.Config, "Decrement")
		self.MinuteDecrementButton.set_Callback(self.CallbackMinuteDecrementButton)
		self.Widgets.append(self.MinuteDecrementButton)
		self.ButtonOrderList.append({"Down":self.MinuteDecrementButton, "Up":self.MinuteIncrementButton})

		if self.Config.AlarmClock.SleepClockEnabled:
			self.EnableButton = widgets.ImageButton(self, EnablePosition, EnableSize, self.Config, "CheckBoxChecked")
		else:
			self.EnableButton = widgets.ImageButton(self, EnablePosition, EnableSize, self.Config, "CheckBoxEmpty")
		self.EnableButton.set_Callback(self.CallbackEnableButton)
		self.Widgets.append(self.EnableButton)
		self.ButtonOrderList.append({"Enter":self.EnableButton})

	def CallbackEnableButton(self):
		self.Config.AlarmClock.SleepClockEnabled = not self.Config.AlarmClock.SleepClockEnabled
		if self.Config.AlarmClock.SleepClockEnabled:
			if self.EnableButton.ButtonType.endswith("Marked"):
				self.EnableButton.set_ButtonType("CheckBoxCheckedMarked")
			else:
				self.EnableButton.set_ButtonType("CheckBoxChecked")
		else:
			if self.EnableButton.ButtonType.endswith("Marked"):
				self.EnableButton.set_ButtonType("CheckBoxEmptyMarked")
			else:
				self.EnableButton.set_ButtonType("CheckBoxEmpty")

	def CallbackHourIncrementButton(self):
		self.SleepHours = self.SleepHours + 1
		if self.SleepHours > 23:
			self.SleepHours = 0
		text = "%02u"%self.SleepHours
		self.HourButton.set_Text(text)
		# timedelta is (days, seconds, microseconds, milliseconds, minutes, hours, weeks)
		self.Config.AlarmClock.SleepClockTime = datetime.datetime.now() + datetime.timedelta(0,0,0,0,self.SleepMinutes,self.SleepHours)

	def CallbackHourDecrementButton(self):
		self.SleepHours = self.SleepHours - 1
		if self.SleepHours < 0:
			self.SleepHours = 23 
		text = "%02u"%self.SleepHours
		self.HourButton.set_Text(text)
		# timedelta is (days, seconds, microseconds, milliseconds, minutes, hours, weeks)
		self.Config.AlarmClock.SleepClockTime = datetime.datetime.now() + datetime.timedelta(0,0,0,0,self.SleepMinutes,self.SleepHours)

	def CallbackMinuteIncrementButton(self):
		self.SleepMinutes = self.SleepMinutes + 1
		if self.SleepMinutes > 59:
			self.SleepMinutes = 0
		text = "%02u"%self.SleepMinutes
		self.MinuteButton.set_Text(text)
		# timedelta is (days, seconds, microseconds, milliseconds, minutes, hours, weeks)
		self.Config.AlarmClock.SleepClockTime = datetime.datetime.now() + datetime.timedelta(0,0,0,0,self.SleepMinutes,self.SleepHours)

	def CallbackMinuteDecrementButton(self):
		self.SleepMinutes = self.SleepMinutes - 1
		if self.SleepMinutes < 0:
			self.SleepMinutes = 59 
		text = "%02u"%self.SleepMinutes
		self.MinuteButton.set_Text(text)
		# timedelta is (days, seconds, microseconds, milliseconds, minutes, hours, weeks)
		self.Config.AlarmClock.SleepClockTime = datetime.datetime.now() + datetime.timedelta(0,0,0,0,self.SleepMinutes,self.SleepHours)

	def HandleEvent(self, event):
		if isinstance(event, events.InfraredEvent):
			if event.button == "Left":
				self.SelectMenuItem(-1)
			elif event.button == "Right":
				self.SelectMenuItem(1)
			elif event.button == "Up":
				if self.ButtonOrderList[self.ButtonOrderActiveItem].has_key("Up"):
					self.ButtonOrderList[self.ButtonOrderActiveItem]["Up"].CallBack()
			elif event.button == "Down":
				if self.ButtonOrderList[self.ButtonOrderActiveItem].has_key("Down"):
					self.ButtonOrderList[self.ButtonOrderActiveItem]["Down"].CallBack()
			elif event.button == "Enter":
				if self.ButtonOrderList[self.ButtonOrderActiveItem].has_key("Enter"):
					self.ButtonOrderList[self.ButtonOrderActiveItem]["Enter"].CallBack()

	def SelectMenuItem(self, relative):
		if self.ButtonOrderActiveItem >= 0 and self.ButtonOrderActiveItem <= len(self.ButtonOrderList) - 1:
			self.ActivateMenuItem(self.ButtonOrderList[self.ButtonOrderActiveItem],False)
		if relative > 0:
			self.ButtonOrderActiveItem = self.ButtonOrderActiveItem + 1
		elif relative <0:
			self.ButtonOrderActiveItem = self.ButtonOrderActiveItem - 1
		if self.ButtonOrderActiveItem < 0:
			self.ButtonOrderActiveItem = len(self.ButtonOrderList) - 1
		elif self.ButtonOrderActiveItem > len(self.ButtonOrderList) - 1:
			self.ButtonOrderActiveItem = 0
		self.ActivateMenuItem(self.ButtonOrderList[self.ButtonOrderActiveItem])


	def ActivateMenuItem(self, MenuItems, Activate=True):
		for MenuItemKey in MenuItems.keys():
			MenuItem = MenuItems[MenuItemKey]
			if Activate:
				if not MenuItem.ButtonType.endswith("Marked"):
					MenuItem.set_ButtonType(MenuItem.ButtonType+"Marked")
			else:
				if MenuItem.ButtonType.endswith("Marked"):
					MenuItem.set_ButtonType(MenuItem.ButtonType[:-6])

#==============================================================================
class ImageArea(widgets.Window):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager, ImageFile, Callback):
		self.evManager = evManager
		widgets.Window.__init__(self, Parent, Position, Size, Config)

		self.Callback = Callback

		self.Image= widgets.ImageButton(self, Position, Size, Config, "")
		self.Image.set_Image(ImageFile)
		self.Image.set_Callback(Callback)
		self.Widgets.append(self.Image)

	def HandleEvent(self, event):
		if isinstance(event, events.InfraredEvent):
			self.Callback()
		elif isinstance(event, events.KeyboardEvent):
			self.Callback()

#==============================================================================
class WakeupImageArea(ImageArea):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		filename = os.path.join(Config.ThemeDir,"Alarm.png")
		ImageArea.__init__(self, Parent, Position, Size, Config, evManager, filename, self.CallbackActivated)

	def CallbackActivated(self):
		ev = events.MainAreaEvent("Status")
		self.evManager.Notify(ev)

#==============================================================================
class SleepImageArea(ImageArea):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager):
		filename = os.path.join(Config.ThemeDir,"Sleep.png")
		ImageArea.__init__(self, Parent, Position, Size, Config, evManager, filename, self.CallbackActivated)

	def CallbackActivated(self):
		ev = events.PowerEvent(False)
		self.evManager.Notify(ev)
		ev = events.MainAreaEvent("Status")
		self.evManager.Notify(ev)

#==============================================================================
class ScreensaverImage(ImageArea):
	MainAreaWidget = True

	def __init__(self, Parent, Position, Size, Config, evManager, imagepath=""):
		filename = os.path.join(Config.ThemeDir,imagepath)
		self.LastImage = None
		self.ImagesInDir = []
		ImageArea.__init__(self, Parent, Position, Size, Config, evManager, filename, self.CallbackActivated)
		self.ChooseImage(self.Config.ScreensaverMode, self.Config.ScreensaverPath)

	def CallbackActivated(self):
		ev = events.ScreensaverEvent(False)
		self.evManager.Notify(ev)

	def HandleEvent(self, event):
		if isinstance(event, events.ScreensaverChangeEvent):
			self.ChooseImage(self.Config.ScreensaverMode, self.Config.ScreensaverPath)

	def ChooseImage(self, Mode, BasePath, CurrentPath=None, CurrentImage=None):
		if self.Config.ScreensaverMode == "random":
			directory = self.Config.ScreensaverPath
			Images = self.RecursiveDirScan(directory,True)
			Image = random.choice(Images)
		elif self.Config.ScreensaverMode == "linear":
			directory = self.Config.ScreensaverPath
			Images = self.RecursiveDirScan(directory,True)
			if self.LastImage == None:
				NewImageIndex = 0
			else:
				try:
					LastImageIndex = Images.index(self.LastImage)
				except:
					LastImageIndex = 0
				NewImageIndex = LastImageIndex + 1
				if NewImageIndex > NewImageIndex:
					NewImageIndex = 0
			Image = Images[NewImageIndex]
		elif self.Config.ScreensaverMode == "randombydir":
			directory = self.Config.ScreensaverPath
			if len(self.ImagesInDir) <= 0:
				dirs = self.RecursiveDirScan(directory,False)
				dir = random.choice(dirs)
				self.ImagesInDir = self.ImageScan(dir)
			Index = random.randint(0,len(self.ImagesInDir)-1)
			Image = self.ImagesInDir.pop(Index)
		else:
			self.logger.fatal("Unknown Screensavermode")
			sys.exit(123)
		self.Image.set_Image(Image)
		self.LastImage = Image

	def RecursiveDirScan(self,directory,scanforfiles):
		Images = []
		Subdirs = []
		if os.path.exists(directory):
			if os.path.isdir(directory):
				if scanforfiles:
					Images = self.ImageScan(directory)
				subdirs = os.listdir(directory)
				subdirs.sort()
				for subdir in subdirs:
					subdirectory = os.path.join(directory,subdir)
					if os.path.isdir(subdirectory):
						Subdirs.append(subdirectory)
						subresults = self.RecursiveDirScan(subdirectory,scanforfiles)
						if scanforfiles:
							Images.extend(subresults)
						else:
							Subdirs.extend(subresults)

		if scanforfiles:
			return Images
		else:
			return Subdirs

	def ImageScan(self, directory):
		IMAGETYPES = [".jpg",".gif",".png",".tif"]
		Images = []
		filenames = os.listdir(directory)
		filenames.sort()
		for filename in filenames:
			completefilename = os.path.join(directory,filename)
			if os.path.isfile(completefilename):
				file, ext = os.path.splitext(completefilename)
				if ext in IMAGETYPES:
					Images.append(completefilename)
		return Images
